/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <gnome.h>
#include <gdbm.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include "pan.h"

#define DEBUG_MSG 0


void
acache_delete (char *message_id)
{
	char path[PATH_MAX];
			
#if DEBUG_MSG
	g_print ("Cache delete: %s\n", message_id);
#endif
	g_snprintf (path, PATH_MAX, "/%s/%s.head", data_dir, message_id);
	unlink (path);
	g_snprintf (path, PATH_MAX, "/%s/%s.body", data_dir, message_id);
	unlink (path);
	g_snprintf (path, PATH_MAX, "/%s/k%s.head", data_dir, message_id);
	unlink (path);
	g_snprintf (path, PATH_MAX, "/%s/k%s.body", data_dir, message_id);
	unlink (path);
}


void
acache_expire (void)
{
	char *p;
	char dirpath[PATH_MAX];
	char path[PATH_MAX];
	DIR *dir_p;
	struct dirent *dirent_p;
	struct stat stat_p;
	time_t t_oldest;
	char n_oldest[NAME_MAX] = "";
	int cache_max, cache_size, n;

	cache_max = gnome_config_get_int_with_default("/Pan/Cache/MaxSize=1242880", NULL);
	g_snprintf (dirpath, PATH_MAX, "/%s", data_dir);
	cache_size = cache_max + 1;
	
	t_oldest = n = 0;
	while (cache_max < cache_size) {
		/* If it's too big and there is more than one entry, delete one */
		if ((cache_max < cache_size) && (n > 1) && t_oldest) { 
			p = (char *)strchr(n_oldest, '>');
			p[1] = '\0';
			acache_delete(n_oldest);
		}
		
		cache_size = t_oldest = n = 0;
		
		/* Get the size and oldest file */
   		dir_p = opendir(dirpath);
		while((dirent_p = readdir(dir_p))) {
			if ((dirent_p->d_name[0] != 'k') && (strstr(dirent_p->d_name, ".head") || strstr(dirent_p->d_name, ".body"))) {
				g_snprintf(path, PATH_MAX, "/%s/%s", data_dir, dirent_p->d_name);
				if ((stat(path, &stat_p) == 0) && (S_ISREG(stat_p.st_mode))) {
					cache_size += stat_p.st_size;
					n++;
					if ((stat_p.st_mtime < t_oldest) || (t_oldest == 0)) {
						strcpy(n_oldest, dirent_p->d_name);
						t_oldest = stat_p.st_mtime;
					} 
				}
			}
		}	
		closedir (dir_p);
#if DEBUG_MSG
		g_print ("Cache size: %d\n", cache_size);
#endif
	}
}


FILE *
acache_open_header (article_data *adata, int flag)
{
	FILE *f;
	char path[PATH_MAX];

	if (flag == ACACHE_KEEP)	
		g_snprintf (path, PATH_MAX, "/%s/k%s.head", data_dir, adata->message_id);
	else
		g_snprintf (path, PATH_MAX, "/%s/%s.head", data_dir, adata->message_id);
	f = fopen (path, "w");
	return f;
}


void
acache_putline_header (FILE *f, char *line)
{
	fputs (line, f);
}


void
acache_close_header (FILE *f)
{
	fclose (f);
}


FILE *
acache_open_body (article_data *adata, int flag)
{
	FILE *f;
	char path[PATH_MAX];
	
	if (flag == ACACHE_KEEP)	
		g_snprintf (path, PATH_MAX, "/%s/k%s.body", data_dir, adata->message_id);
	else
		g_snprintf (path, PATH_MAX, "/%s/%s.body", data_dir, adata->message_id);

	f = fopen (path, "w");
	return f;
}


void
acache_putline_body (FILE *f, char *line)
{
	fputs (line, f);
}


void
acache_close_body (FILE *f)
{
	int cache_max;

	fclose(f);
	
	cache_max = gnome_config_get_int ("/Pan/Cache/MaxSize=5242880");
	/* This will expire some, if necessary */
	acache_expire();
}


int
acache_hit_body (article_data *adata)
{
	char path[PATH_MAX];
	struct stat st;
	int rc;
	
	g_snprintf(path, PATH_MAX, "/%s/%s.body", data_dir, adata->message_id);
	if ((rc = stat(path, &st)) != 0) {
		g_snprintf(path, PATH_MAX, "/%s/k%s.body", data_dir, adata->message_id);
		rc = stat(path, &st);
	}
#if DEBUG_MSG
	if (rc == 0)
		g_print ("Cache hit (body) for: %s\n", adata->message_id);
#endif
	return rc;
}


int
acache_hit_header (article_data *adata)
{
	char path[PATH_MAX];
	struct stat st;
	int rc;

	g_snprintf(path, PATH_MAX, "/%s/%s.head", data_dir, adata->message_id);
	if ((rc = stat(path, &st)) != 0) {
		g_snprintf(path, PATH_MAX, "/%s/k%s.head", data_dir, adata->message_id);
		rc = stat(path, &st);
	}
#if DEBUG_MSG
	if (rc == 0)
		g_print ("Cache hit (header) for: %s\n", adata->message_id);
#endif
	return rc;
}


int
acache_load_body (article_data *adata)
{
	FILE *f;
	char path[PATH_MAX];
	char buffer[1024];

	g_free(adata->body);
	adata->body = NULL;
	g_snprintf(path, PATH_MAX, "/%s/%s.body", data_dir, adata->message_id);
	if ((f = fopen(path, "r")) == NULL) {
		g_snprintf(path, PATH_MAX, "/%s/k%s.body", data_dir, adata->message_id);
		if ((f = fopen(path, "r")) == NULL)
			return FALSE;
	}
	
	adata->body = g_strdup("");
	while (fgets(buffer, 1024, f)) {
		if (adata->body) {
			adata->body = g_realloc(adata->body, strlen(adata->body) + strlen(buffer) + 1);
			adata->body = strcat(adata->body, buffer);
		}
	}
	fclose(f);
	return TRUE;
}


int
acache_load_header (article_data *adata)
{
	FILE *f;
	char path[PATH_MAX];
	char buffer[1024];

	g_free(adata->header);
	adata->header = NULL;

	g_snprintf(path, PATH_MAX, "/%s/%s.head", data_dir, adata->message_id);
	if ((f = fopen(path, "r")) == NULL) {
		g_snprintf(path, PATH_MAX, "/%s/k%s.head", data_dir, adata->message_id);
		if ((f = fopen(path, "r")) == NULL)
			return FALSE;
	}
	
	adata->header = g_strdup("");
	while (fgets(buffer, 1024, f)) {
		if (adata->header) {
			adata->header = g_realloc(adata->header, strlen(adata->header) + strlen(buffer) + 1);
			adata->header = strcat(adata->header, buffer);
		}
	}
	fclose(f);
	return TRUE;
}


void
acache_load (article_data *adata)
{
	acache_load_header(adata);
	acache_load_body(adata);
}
