/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <string.h>
#include <gnome.h>
#include <ctype.h>
#include "pan.h"

#include "xpm/book_open.xpm"
#include "xpm/book_closed.xpm"
#include "xpm/mini_page.xpm"
#include "xpm/greencheck.xpm"
#include "xpm/bluecheck.xpm"
#include "xpm/x.xpm"
#include "xpm/binary.xpm"
#include "xpm/binary_complete.xpm"
#include "xpm/binary_incomplete.xpm"

/* Global Variables */
static GnomePixmap *articlelist_closed_pixmap;
static GnomePixmap *articlelist_open_pixmap;
static GnomePixmap *decode_queue_pixmap;
static GnomePixmap *download_queue_pixmap;
static GnomePixmap *article_downloaded_pixmap;
static GnomePixmap *mp_complete_pixmap;
static GnomePixmap *mp_incomplete_pixmap;
static GnomePixmap *error_pixmap;
static GnomePixmap *article_read_pixmap;
static GnomePixmap *binary_decoded_pixmap;

GdkColormap *cmap;
GdkColor new_color;
GdkColor read_color;
GdkColor unread_color;

article_data *current_article = NULL;
GtkWidget *article_ctree_menu;
static GtkWidget *find_dialog;
int sort_style;

static GnomeUIInfo articlelist_menu_popup[] = {
	{ GNOME_APP_UI_ITEM, _("Decode Message"), _("Decode messages."), articlelist_selected_decode,
	  NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'D', GDK_CONTROL_MASK, NULL },
	{ GNOME_APP_UI_ITEM, _("Delete Message"), _("Delete message from database."), article_delete },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, _("Mark as Read"), _("Marks current message as read."), articlelist_selected_mark_read },
	{ GNOME_APP_UI_ITEM, _("Mark as Unread"), _("Marks the current message as unread."), articlelist_selected_mark_unread },
	GNOMEUIINFO_END
};


/**********************************************************
 * The following stuff implements the "find" functionality
 **********************************************************/

static GtkCTreeNode *next_pos = NULL;

void
find_dialog_destroy_cb (void)
{
	gtk_widget_destroy(find_dialog);
	find_dialog = NULL;
}


int
find_compare_func (article_data *adata, char *p)
{
	if (adata && adata->subject) {
		if (strstr(adata->subject, p)) {
			return 0;
		}
	}
	return -1;
}


void
find_article (gchar *p)
{
	GtkCTreeNode *node, *n;
	static char *find_text = NULL;

	if (p) {
		g_free (find_text);
		find_text = p;
	}

	if (find_text) {
		node = gtk_ctree_find_by_row_data_custom(GTK_CTREE(Pan.article_ctree), next_pos, find_text, 
							 (GCompareFunc)find_compare_func);
		if (node) {
			gtk_clist_freeze (GTK_CLIST(Pan.article_ctree));
			gtk_ctree_unselect_recursive (GTK_CTREE(Pan.article_ctree), NULL);
			gtk_ctree_select (GTK_CTREE(Pan.article_ctree), node);
			/* Make sure the found node is not in a collapsed branch */
			for (n = GTK_CTREE_ROW(node)->parent; n; n = GTK_CTREE_ROW(n)->parent)
				gtk_ctree_expand(GTK_CTREE(Pan.article_ctree), n);
			gtk_ctree_node_moveto (GTK_CTREE(Pan.article_ctree), node, 0, 0, 0);
			gtk_clist_thaw (GTK_CLIST(Pan.article_ctree));
			
			/* Find next node, paying attention to recursion */
			if (GTK_CTREE_ROW(node)->children) {
				next_pos = GTK_CTREE_ROW(node)->children;
			} else {
				for (n = node; n; n = GTK_CTREE_ROW(n)->parent) {
					if (GTK_CTREE_ROW(n)->sibling) {
						next_pos = GTK_CTREE_ROW(n)->sibling;
						break;
					}
				}
			}
			/***/
		} else {
			next_pos = NULL;
		}
	}
}


void
find_dialog_clicked_cb (GnomeDialog *dialog, int button, GtkWidget *find_entry)
{
	gchar *p;

	switch (button) {
	case 0:					/* OK button */
		p = g_strdup (gtk_entry_get_text(GTK_ENTRY(find_entry)));
		if (strlen (p) > 0) {
			find_article (p);
		}
	default:				/* Cancel Button */
		find_dialog_destroy_cb();
		break;
	}
}


void
articlelist_find_next_cb (void)
{
	find_article (NULL);
}


void
articlelist_find_text_cb (void)
{
	GtkWidget *hbox, *label, *find_entry;

	/* There can be only one! (find window) */
	if (find_dialog) {
		gdk_window_raise (GTK_WIDGET(find_dialog)->window);
		return;
	}
	
	find_dialog = gnome_dialog_new (_("Find Text"),
					GNOME_STOCK_BUTTON_OK,
					GNOME_STOCK_BUTTON_CANCEL, NULL);

	hbox = gtk_hbox_new (FALSE, 0);
	label = gtk_label_new (_("Find:"));
	find_entry = gtk_entry_new ();	
	
	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 5);
	gtk_box_pack_end (GTK_BOX(hbox), find_entry, TRUE, TRUE, 5);
	
	gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(find_dialog)->vbox), hbox);
	
	gtk_signal_connect (GTK_OBJECT(find_dialog), "clicked",
			    GTK_SIGNAL_FUNC(find_dialog_clicked_cb),
			    find_entry);
	gtk_signal_connect (GTK_OBJECT(find_dialog), "destroy",
			    GTK_SIGNAL_FUNC(find_dialog_destroy_cb), NULL);

	gtk_widget_grab_focus (find_entry);
	gui_popup_draw (find_dialog);
}


/*--------------------------------------------------------------------
 * free an article list (and all threads)
 *--------------------------------------------------------------------*/
void
articlelist_free (GSList *alist)
{
	GSList *p;
	article_data *adata;

	for (p = alist; p != NULL; p = p->next) {
/*		g_assert (p->data); */
		adata = p->data;

		if (adata->threads)
			articlelist_free (adata->threads);
		article_free (adata);
		p->data = NULL;
	}

	g_slist_free (alist);
}


void
articlelist_get_all (void)
{
	if (Pan.active_group)
		queue_add_articlelist_download (Pan.active_group,
						QUEUE_ARTICLELIST_DOWNLOAD_ALL |
						QUEUE_FOREGROUND);
}


void
articlelist_get_new (void)
{
	if (Pan.active_group)
		queue_add_articlelist_download (Pan.active_group,
						QUEUE_ARTICLELIST_DOWNLOAD_NEW |
						QUEUE_FOREGROUND);
}


/*--------------------------------------------------------------------
 * clear the ctree of all messages
 *--------------------------------------------------------------------*/
void
articlelist_clear (void)
{
	pan_lock();
	gtk_clist_freeze (GTK_CLIST(Pan.article_ctree));
	gtk_clist_clear (GTK_CLIST(Pan.article_ctree));
	gtk_clist_thaw (GTK_CLIST(Pan.article_ctree));
	pan_unlock();
}

/*--------------------------------------------------------------------
 * load article listing
 *--------------------------------------------------------------------*/
void
articlelist_load (queue_data *qdata)
{
	group_data *gdata = NULL;
	GSList *alist;

	/* Clear the currently displayed articlelist, if any */
	if (Pan.active_group) {
		articlelist_clear ();
		articlelist_free (Pan.active_group->articlelist);
		Pan.active_group->articlelist = NULL;
	}

	gdata = qdata->gdata;

	/* step 1: load article headers */
	if ((alist = db_load_articles (gdata)) == NULL) {
		pan_lock ();
		choose_download_dialog (gdata);
		pan_unlock ();
	} 
	else {
		grouplist_update_row (gdata);
//		gui_set_info_va (_("Loaded %d article headers"), group_get_attrib_i (gdata, "Total"));
		log_add_va (_("Loaded %d article headers for group %s"), group_get_attrib_i (gdata, "Total"), gdata->name);

		/* step 2: thread article list */
		gui_set_status (_("Threading article list"));

		alist = thread_list_by_multipart (alist, gdata);
		alist = thread_list_by_subject (alist, gdata);
		alist = thread_list_by_reference (alist, gdata);

		gdata->articlelist = alist;

		gui_set_status (_("Sorting article list"));
		articlelist_sort (gdata);

		/* step 3: display article list */
		gui_set_status (_("Displaying article list"));
		articlelist_update ((group_data *) gdata);
	}
}


/*--------------------------------------------------------------------
 * this function removes the STATE_NEW flag from every article in 
 * the list, called when 'get new articles' is run, to mark all as old
 *--------------------------------------------------------------------*/
void
articlelist_not_new (GSList *l)
{
	for (; l; l = l->next) {
		((article_data *)l->data)->state &= ~STATE_NEW;
		if (((article_data *)l->data)->threads) {
			articlelist_not_new (((article_data *)l->data)->threads);
		}
	}
}

/*--------------------------------------------------------------------
 * process the selected articles
 *--------------------------------------------------------------------*/
void
articlelist_process_selected (int type)
{
	GList *p;
	article_data *adata;

	for (p = GTK_CLIST(Pan.article_ctree)->selection; p; p = p->next) {
		adata = (article_data *) gtk_ctree_node_get_row_data(GTK_CTREE(Pan.article_ctree), p->data);
		if (adata) {
			switch (type) {
			case 2:
				queue_add_article_decode(adata, QUEUE_ARTICLE_DECODE | QUEUE_BACKGROUND);
				adata->state |= STATE_DECODE_QUEUED;
				/* Calling article_update here is too slow */
				articlelist_update_node (adata);
				break;
			case 3: /* mark read */
				article_mark_read (adata);
				articlelist_update_node (adata);
				break;
			case 4: /* mark unread */
				article_mark_unread (adata);
				articlelist_update_node (adata);
				break;
			case 5: /* delete */
				article_delete ();
			default:
				break;
			}
		}
	}
	gtk_ctree_unselect_recursive (GTK_CTREE(Pan.article_ctree), NULL);
}


void
article_ctree_clean (void)
{
	gtk_clist_clear (GTK_CLIST (Pan.article_ctree));
	if (Pan.active_group) {
		group_clean (Pan.active_group);
		Pan.active_group = NULL;
	}
}


inline void
articlelist_select_all (void)
{
	gtk_ctree_select_recursive (GTK_CTREE(Pan.article_ctree), NULL);
}


GtkCTreeNode*
add_article_to_ctree (GtkCTreeNode *parent, article_data *adata)
{
	char *text[4];
	GtkCTreeNode *node;

	text[0] = adata->subject;
	text[1] = g_strdup_printf ("%d", adata->linecount);
	text[2] = g_strdup (ctime((const time_t *) &adata->date));
	text[3] = adata->author;

	if (adata->state & STATE_ERROR) {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
					     NULL, text, 5,
					     error_pixmap->pixmap,
					     error_pixmap->mask,
					     error_pixmap->pixmap,
					     error_pixmap->mask, FALSE, FALSE);
	} else if (adata->state & STATE_DECODED) {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
					     NULL, text, 5,
					     binary_decoded_pixmap->pixmap,
					     binary_decoded_pixmap->mask,
					     binary_decoded_pixmap->pixmap,
					     binary_decoded_pixmap->mask, FALSE,
					     FALSE);
	} else if (adata->state & STATE_READ) {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
					     NULL, text, 5,
					     article_read_pixmap->pixmap,
					     article_read_pixmap->mask,
					     article_read_pixmap->pixmap,
					     article_read_pixmap->mask, FALSE,
					     FALSE);
	} else if (adata->state & STATE_DOWNLOAD_QUEUED) {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
					     NULL, text, 5,
					     download_queue_pixmap->pixmap,
					     download_queue_pixmap->mask,
					     download_queue_pixmap->pixmap,
					     download_queue_pixmap->mask, FALSE,
					     FALSE);
	} else if (adata->state & STATE_DECODE_QUEUED) {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
					     NULL, text, 5,
					     decode_queue_pixmap->pixmap,
					     decode_queue_pixmap->mask,
					     decode_queue_pixmap->pixmap,
					     decode_queue_pixmap->mask, FALSE, FALSE);
	} else if (adata->state & STATE_MULTIPART_ALL) {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
					     NULL, text, 5,
					     mp_complete_pixmap->pixmap,
					     mp_complete_pixmap->mask,
					     mp_complete_pixmap->pixmap,
					     mp_complete_pixmap->mask, FALSE, FALSE);
	} else if (adata->state & STATE_MULTIPART_SOME) {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
					     NULL, text, 5,
					     mp_incomplete_pixmap->pixmap,
					     mp_incomplete_pixmap->mask,
					     mp_incomplete_pixmap->pixmap,
					     mp_incomplete_pixmap->mask, FALSE,
					     FALSE);
	} else {
		node = gtk_ctree_insert_node(GTK_CTREE(Pan.article_ctree), parent,
					     NULL, text, 5, NULL, NULL, NULL, NULL,
					     FALSE, FALSE);
	}

	/* Set text color */ 
	if ((adata->state & STATE_READ) || (adata->state & STATE_DECODED)) {
		gtk_ctree_node_set_foreground (GTK_CTREE(Pan.article_ctree), node, &read_color);
	} else if (adata->state & STATE_NEW) {
		gtk_ctree_node_set_foreground(GTK_CTREE(Pan.article_ctree), node, &new_color);
	} else { 
		gtk_ctree_node_set_foreground(GTK_CTREE(Pan.article_ctree), node, &unread_color);
	}

	gtk_ctree_node_set_row_data (GTK_CTREE(Pan.article_ctree), node, (article_data *) adata);
	adata->node = node;

	g_free (text[1]);
	g_free (text[2]);

	return node;
}


int
build_article_ctree_recursive (GtkCTreeNode *parent, GSList* list)
{
	GSList *pos = list;
	article_data *data;
	GtkCTreeNode *sibling;
	int count = 0;

	while (pos) {
		data = (article_data *) pos->data;
		pos = pos->next;
		sibling = add_article_to_ctree (parent, data);
		++count;
		if (data->threads)
			count += build_article_ctree_recursive (sibling, data->threads);
	}
	return count;
}


void
articlelist_update (group_data *gdata)
{
	char *text[4];
	GtkCTreeNode *parent;

	if (gdata == NULL)
		return;

	text[0] = g_strdup (gdata->name);
	text[1] = text[2] = text[3] = NULL;

	gui_set_status (_("Updating article display"));
	articlelist_clear();

	pan_lock ();

	gtk_clist_freeze (GTK_CLIST(Pan.article_ctree));
	parent = gtk_ctree_insert_node (GTK_CTREE(Pan.article_ctree), NULL, NULL,
					text, 5, articlelist_closed_pixmap->pixmap,
					articlelist_closed_pixmap->mask,
					articlelist_open_pixmap->pixmap,
					articlelist_open_pixmap->mask, FALSE, TRUE);

	build_article_ctree_recursive (parent, gdata->articlelist);
	gtk_clist_thaw (GTK_CLIST (Pan.article_ctree));

//	if (Pan.active_group && (Pan.active_group != gdata))
//		group_clean (Pan.active_group);

	Pan.active_group = gdata;

	pan_unlock ();
}



void 
articlelist_update_node (article_data * adata)
{
	GtkCTreeNode *node = NULL;

	g_assert (adata && adata->message_id);

	if (adata->node) {
		node = adata->node;
	} else {
		node = gtk_ctree_find_by_row_data (GTK_CTREE(Pan.article_ctree), NULL, adata);
	}

	if (node) {
		if (adata->state & STATE_DOWNLOAD_QUEUED) {
			gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node,
						adata->subject, 5,
						download_queue_pixmap->pixmap,
						download_queue_pixmap->mask,
						download_queue_pixmap->pixmap,
						download_queue_pixmap->mask,
						GTK_CTREE_ROW(node)->is_leaf,
						GTK_CTREE_ROW(node)->expanded);
		} else if (adata->state & STATE_DECODE_QUEUED) {
			gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node,
						adata->subject, 5,
						decode_queue_pixmap->pixmap,
						decode_queue_pixmap->mask,
						decode_queue_pixmap->pixmap,
						decode_queue_pixmap->mask,
						GTK_CTREE_ROW(node)->is_leaf,
						GTK_CTREE_ROW(node)->expanded);
		} else if (adata->state & STATE_ERROR) {
			gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node,
						adata->subject, 5,
						error_pixmap->pixmap,
						error_pixmap->mask,
						error_pixmap->pixmap,
						error_pixmap->mask,
						GTK_CTREE_ROW(node)->is_leaf,
						GTK_CTREE_ROW(node)->expanded);
		} else if (adata->state & STATE_DECODED) {
			gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node,
						adata->subject, 5,
						binary_decoded_pixmap->pixmap,
						binary_decoded_pixmap->mask,
						binary_decoded_pixmap->pixmap,
						binary_decoded_pixmap->mask,
						GTK_CTREE_ROW(node)->is_leaf,
						GTK_CTREE_ROW(node)->expanded);
		} else if (adata->state & STATE_READ) {
			gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node,
						adata->subject, 5,
						article_read_pixmap->pixmap,
						article_read_pixmap->mask,
						article_read_pixmap->pixmap,
						article_read_pixmap->mask,
						GTK_CTREE_ROW(node)->is_leaf,
						GTK_CTREE_ROW(node)->expanded);
		} else {
			gtk_ctree_set_node_info(GTK_CTREE(Pan.article_ctree), node,
						adata->subject, 5, NULL, NULL, NULL,
						NULL, GTK_CTREE_ROW(node)->is_leaf,
						GTK_CTREE_ROW(node)->expanded);
		}
		
		/* Set text color */ 
		if ((adata->state & STATE_READ) || (adata->state & STATE_DECODED)) {
			gtk_ctree_node_set_foreground (GTK_CTREE(Pan.article_ctree), node, &read_color);
		} else if (adata->state & STATE_NEW) {
			gtk_ctree_node_set_foreground (GTK_CTREE(Pan.article_ctree), node, &new_color);
		} else { 
			gtk_ctree_node_set_foreground (GTK_CTREE(Pan.article_ctree), node, &unread_color);
		}
	}
}


void articlelist_selected_download (void) { articlelist_process_selected (0); }
void articlelist_selected_decode (void) { articlelist_process_selected (2); }
void articlelist_selected_mark_read (void) { articlelist_process_selected (3); }
void articlelist_selected_mark_unread (void) { articlelist_process_selected (4); }
void articlelist_selected_delete (void) { articlelist_process_selected (5); }


int
multipart_sortfunc (article_data *a, article_data *b)
{
	if (a->part > b->part)
		return 1;
	else
		return -1;
}


/*--------------------------------------------------------------------
 *
 *--------------------------------------------------------------------*/
int 
mp_strcmp (char *mp, char *ph)
{
	int i = 0;
	int j = 0;

	while ((mp[i] != '\0') && (ph[j] != '\0')) {
		if (((mp[i] == '(') && (ph[j] == '(')) ||
		    ((mp[i] == '[') && (ph[j] == '['))) {
			++j;
			++i;
			if (isdigit(ph[j]) && isdigit(mp[i])) {
				for (; isdigit(ph[j]); ++j);
				for (; isdigit(mp[i]); ++i);
			}
		}
		else if (mp[i] == ph[j]) {
			i++;
			j++;
		}
		else
			return 1;
	}
	return 0;
}


GSList *
thread_article_by_multipart (GSList *list, article_data *adata)
{
	GSList *p;
	article_data *data;

	for (p = list; p; p = p->next) {
		data = p->data;
		if ((data->parts >= 2) && 
		    (data->parts == adata->parts) && 
		    (data->part < adata->part))
		{
			if (!mp_strcmp (data->subject, adata->subject)) {
				list = g_slist_remove (list, adata);
				if (adata->threads) {
					data->threads = g_slist_concat (data->threads, adata->threads);
					data->nthreads += adata->nthreads;
					adata->threads = NULL;
					adata->nthreads = 0;
				}
				data->threads = g_slist_prepend (data->threads, adata);
				data->nthreads++;
				return list;
			}
		}
	}

	return list;
}


int
thread_article_by_reference (GSList *list, article_data* data)
{
	GSList *pos;
	article_data *adata;

	pos = list;

	while (pos) {
		adata = (article_data *) pos->data;
		pos = pos->next;
		if (adata->references && (strcmp(data->message_id, adata->message_id))) {
			if (!strcmp(strrchr(data->references, '<'), adata->message_id)) {
				/* we found our home! Thread it here */
				adata->threads = g_slist_prepend (adata->threads, data);
				adata->nthreads++;
				return 1;
			}
			if (adata->threads) {
				if (thread_article_by_reference(adata->threads, data))
					return 1;
			}
		}
	} /* end while loop */

	return 0;
}


int
thread_article_by_subject (GSList *list, article_data *data)
{
	GSList *tmp;
	article_data *adata, *min = NULL;

	tmp = list;

	while (tmp) {
		adata = (article_data *) tmp->data;
		tmp = tmp->next;
		if (adata != data) {
			if (!strcmp(adata->subject, data->subject + 4)) {
				/* Found  "Re: name" and "name" */
				adata->threads = g_slist_prepend (adata->threads, data);
				adata->nthreads++;
				return 1;
			}
			else if (!strcasecmp(adata->subject, data->subject)) {
				if (adata->threads) {
					adata->threads = g_slist_prepend (adata->threads, data);
					adata->nthreads++;
					return 1;
				}
				else {
					if (min == NULL)
						min = (adata->date < data->date) ? adata : NULL;
					else if (adata->date < min->date)
						min = adata;
				}
			}
		}
	}
	if (min) {
		min->threads = g_slist_prepend (min->threads, data);
		min->nthreads++;
		return 1;
	}
	else
		return 0;
}


GSList *
thread_list_by_multipart (GSList *list, group_data *gdata)
{
	article_data *adata;
	GSList *p;
	gfloat count, total;

	g_assert (list);

	count = 0;
	total = group_get_attrib_i (gdata, "Total") * 3;

	p = list;
	while (p) {
		gui_set_progress (count++ / total);
		adata = (article_data *) p->data;
		p = p->next;
		if (adata->parts >= 2)
			/* found a multipart, thread it */
			list = thread_article_by_multipart (list, adata);
	}

	for (p = list; p; p = p->next) {
		adata = (article_data *) p->data;
		if (adata->parts >= 2) {
			if ((adata->nthreads == (adata->parts - 1)) || (adata->nthreads == adata->parts)) {
				adata->threads = g_slist_sort (adata->threads, (GCompareFunc)multipart_sortfunc);
				adata->state |= STATE_MULTIPART_ALL;
			}
			else {
				adata->state |= STATE_MULTIPART_SOME;
			}
			/* FIXME: This is probably a good place to do something about
			 * multiparts with duplicate parts */
		}
	}
	return list;
}


GSList *
thread_list_by_reference (GSList* list, group_data* gdata)
{
	article_data *adata;
	GSList *tmp;
	gfloat count, total;

	count = group_get_attrib_i (gdata, "Total") * 2;
	total = group_get_attrib_i (gdata, "Total") * 3;
	tmp = list;

	while (tmp) {
		gui_set_progress (count++ / total);
		adata = (article_data *) tmp->data;
		tmp = tmp->next;
		if (adata->threads)
			adata->threads = thread_thread_by_reference (adata->threads);
	}
	gui_set_progress (1);
	return list;
}


GSList*
thread_list_by_subject (GSList* list, group_data* gdata)
{
	article_data *adata;
	GSList *pos, *last;
	int rc;
	gfloat count, total;

	g_assert (list);

	count = group_get_attrib_i (gdata, "Total");
	total = group_get_attrib_i (gdata, "Total") * 3;

	pos = list;

	while (pos) {
		gui_set_progress (count++ / total);
		adata = (article_data *) pos->data;
		last = pos;
		pos = pos->next;
		if (!strncasecmp ("re:", adata->subject, 3)) {
			rc = thread_article_by_subject (list, adata);
			if (rc) {
				list = g_slist_remove (list, adata);
			}
		}
	}
	return list;
}


GSList *
thread_thread_by_reference (GSList* list)
{
	article_data *adata;
	GSList *tmp;
	int rc;

	tmp = list;
	while (tmp) {
		adata = (article_data *) tmp->data;
		tmp = tmp->next;
		if (adata->references && adata->references[0] == '<') {
			rc = thread_article_by_reference (list, adata);
			if (rc)
				list = g_slist_remove (list, adata);
		}
	}
	return list;
}


void
article_ctree_button_press (GtkWidget* widget, GdkEventButton* bevent)
{
	int row;
	int column;
	article_data *adata;

	if (bevent->button != 1)
		gtk_signal_emit_stop_by_name (GTK_OBJECT(widget),
					      "button_press_event");

	if ((bevent->button) == 1 && (bevent->type == GDK_2BUTTON_PRESS))
	{
		gtk_signal_emit_stop_by_name (GTK_OBJECT(widget),
					      "button_press_event");
		gtk_clist_get_selection_info (GTK_CLIST(widget), bevent->x,
					      bevent->y, &row, &column);
		adata = (article_data *) gtk_clist_get_row_data (GTK_CLIST(widget), row);
		if (adata) {
			current_article = adata;
			if (adata->body == NULL) {
				queue_add_article_download (adata,
							    QUEUE_ARTICLE_DOWNLOAD |
							    QUEUE_FOREGROUND);
				adata->state |= STATE_DOWNLOAD_QUEUED;
				articlelist_update_node (adata);
			}
			else text_update (adata);
		}
		/* if in notebook layout, flip to the message tab */
		gui_page_set (MESSAGE_PAGE, text);

	}
	else if (bevent->button == 3)
		gtk_menu_popup (GTK_MENU(article_ctree_menu), NULL, NULL, NULL,
				NULL, bevent->button, bevent->time);
}


int compare_subject_ascending (article_data *a, article_data *b) { 
	return (strcasecmp(a->subject, b->subject)); 
}

int compare_subject_descending (article_data *a, article_data *b) { 
	return (strcasecmp(b->subject, a->subject)); 
}

int compare_lines_ascending (article_data *a, article_data *b) {
	if (a->linecount > b->linecount)
		return 1;
	else
		return -1;
}

int compare_lines_descending (article_data *a, article_data *b) {
	if (b->linecount > a->linecount)
		return 1;
	else
		return -1;
}

int compare_author_ascending (article_data *a, article_data *b) { 
	return (strcasecmp(a->author, b->author));
}
 
int compare_author_descending (article_data *a, article_data *b) {
	return (strcasecmp(b->author, a->author));
}

int compare_date_ascending (article_data *a, article_data *b) {
	if (a->date > b->date)
		return 1;
	else
		return -1;
}

int compare_date_descending (article_data *a, article_data *b) {
	if (b->date > a->date)
		return 1;
	else
		return -1;
}


void
articlelist_sort (group_data *gdata)
{
	sort_style = group_get_attrib_i (gdata, "sort_style");

	if (!gdata)
		return;

	switch (sort_style) {
	case SORT_SUBJECT_ASC:
		gdata->articlelist = g_slist_sort (gdata->articlelist,
						   (GCompareFunc) compare_subject_ascending);
		break;
	case SORT_SUBJECT_DESC:
		gdata->articlelist = g_slist_sort (gdata->articlelist,
						   (GCompareFunc) compare_subject_descending);
		break;
	case SORT_LINES_ASC:
		gdata->articlelist = g_slist_sort (gdata->articlelist,
						   (GCompareFunc) compare_lines_ascending);
		break;
	case SORT_LINES_DESC:
		gdata->articlelist = g_slist_sort (gdata->articlelist,
						   (GCompareFunc) compare_lines_descending);
		break;
	case SORT_DATE_ASC:
		gdata->articlelist = g_slist_sort (gdata->articlelist,
						   (GCompareFunc) compare_date_ascending);
		break;
	case SORT_DATE_DESC:
		gdata->articlelist = g_slist_sort (gdata->articlelist,
						   (GCompareFunc) compare_date_descending);
		break;
	case SORT_AUTHOR_ASC:
		gdata->articlelist = g_slist_sort (gdata->articlelist,
						   (GCompareFunc) compare_author_ascending);
		break;
	case SORT_AUTHOR_DESC:
		gdata->articlelist = g_slist_sort (gdata->articlelist,
						   (GCompareFunc) compare_author_descending);
		break;
	default:
		g_warning ("Reached default in articlelist_sort()\n");
		return;
	}
}


/*---[ click_articlelist_column ]-------------------------------------
 * user has clicked one of the column titles, it's our duty to sort
 * the elements of the CList to reflect their clickage.
 *--------------------------------------------------------------------*/
void 
click_articlelist_column (GtkCList *clist, int n)
{
	if (Pan.active_group == NULL)
		return;

	sort_style = group_get_attrib_i (Pan.active_group, "sort_style");

	if (sort_style > 3) {
		n += 4;
		if (sort_style == n) 
			sort_style = (n - 4);
		else
			sort_style = (n-4);
	} 
	else {
		if (sort_style == n)
			sort_style = n + 4;
		else
			sort_style = n;
	}

        switch (sort_style) {
        case SORT_SUBJECT_ASC:
                Pan.active_group->articlelist = g_slist_sort (Pan.active_group->articlelist,
                                                              (GCompareFunc) compare_subject_ascending);
                break;
        case SORT_SUBJECT_DESC:
                Pan.active_group->articlelist = g_slist_sort (Pan.active_group->articlelist,
                                                              (GCompareFunc) compare_subject_descending);
                break;
        case SORT_LINES_ASC:
                Pan.active_group->articlelist = g_slist_sort (Pan.active_group->articlelist,
                                                              (GCompareFunc) compare_lines_ascending);
                break;
        case SORT_LINES_DESC:
                Pan.active_group->articlelist = g_slist_sort (Pan.active_group->articlelist,
                                                              (GCompareFunc) compare_lines_descending);
                break;
        case SORT_DATE_ASC:
                Pan.active_group->articlelist = g_slist_sort (Pan.active_group->articlelist,
                                                              (GCompareFunc) compare_date_ascending);
                break;
        case SORT_DATE_DESC:
                Pan.active_group->articlelist = g_slist_sort (Pan.active_group->articlelist,
                                                              (GCompareFunc) compare_date_descending);
                break;
        case SORT_AUTHOR_ASC:
                Pan.active_group->articlelist = g_slist_sort (Pan.active_group->articlelist,
                                                              (GCompareFunc) compare_author_ascending);
                break;
        case SORT_AUTHOR_DESC:
                Pan.active_group->articlelist = g_slist_sort (Pan.active_group->articlelist,
                                                              (GCompareFunc) compare_author_descending);
                break;
        default:
                g_warning ("Reached default in click_articlelist_column()\n");
                return;
        }

        group_set_attrib_i (Pan.active_group, "sort_style", sort_style);

        articlelist_update (Pan.active_group);

}


/*--------------------------------------------------------------------
 * generate the listing of articles, for the "Articles" tab
 *--------------------------------------------------------------------*/
GtkWidget*
create_articlelist_ctree (void)
{
	GtkWidget *s_window;

	static char *titles[] = {
		_("Subject"),
		_("Lines"),
		_("Date"),
		_("Author"),
	};

	articlelist_closed_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(book_closed_xpm);
	articlelist_open_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(book_open_xpm);
	article_read_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(mini_page_xpm);
	decode_queue_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(bluecheck_xpm);
	download_queue_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(greencheck_xpm);
	article_downloaded_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(mini_page_xpm);
	error_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(x_xpm);
	binary_decoded_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(binary_xpm);
	mp_complete_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(binary_complete_xpm);
	mp_incomplete_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d(binary_incomplete_xpm);

	cmap = gdk_colormap_get_system ();
	unread_color.red = 0;
	unread_color.green = 0;
	unread_color.blue = 0x9999;
	new_color.red = 0xffff;
	new_color.green = 0;
	new_color.blue = 0;
	read_color.red = 0;
	read_color.green = 0;
	read_color.blue = 0;
	if (!gdk_color_alloc(cmap, &unread_color)) {
		g_error("couldn't allocate color");
	}
	if (!gdk_color_alloc(cmap, &new_color)) {
		g_error("couldn't allocate color");
	}
	if (!gdk_color_alloc(cmap, &read_color)) {
		g_error("couldn't allocate color");
	}

	Pan.article_ctree = gtk_ctree_new_with_titles (4, 0, titles);
	gtk_ctree_set_line_style (GTK_CTREE(Pan.article_ctree),
				  GTK_CTREE_LINES_DOTTED);
	gtk_widget_show (Pan.article_ctree);

	s_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(s_window),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER(s_window), Pan.article_ctree);
	gtk_widget_show (s_window);

	gtk_signal_connect (GTK_OBJECT(Pan.article_ctree), "button_press_event",
			    GTK_SIGNAL_FUNC(article_ctree_button_press), NULL);

	gtk_signal_connect (GTK_OBJECT(Pan.article_ctree), "click_column",
			    GTK_SIGNAL_FUNC(click_articlelist_column), NULL);

	gtk_clist_set_selection_mode (GTK_CLIST(Pan.article_ctree),
				      GTK_SELECTION_EXTENDED);

/*
  FIXME: nice to have the date column right justified, but then it doesn't
   seem to allow for resizing the columns next to it... strange
*/
	gtk_clist_set_column_justification (GTK_CLIST (Pan.article_ctree), 2,
					    GTK_JUSTIFY_RIGHT);
	
	gtk_clist_set_column_width (GTK_CLIST(Pan.article_ctree), 0, 400);
	gtk_clist_set_column_auto_resize (GTK_CLIST(Pan.article_ctree), 1, TRUE);
	gtk_clist_set_column_auto_resize (GTK_CLIST(Pan.article_ctree), 2, TRUE);

	/* Create the Popup Menu While we are here */
	article_ctree_menu = gnome_popup_menu_new (articlelist_menu_popup);

	return s_window;
}
