/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <string.h>
#include <gnome.h>
#include <errno.h>
#include "pan.h"
#include "xpm/disk.xpm"

/*
 * globals 
 */
int group_mode = GROUP_ALL;
GtkWidget *groups_clist_menu;
static GnomePixmap *disk_pixmap = NULL;

static GnomeUIInfo groups_menu_popup[] = {
	{ GNOME_APP_UI_ITEM, _("Subscribe to Selected"), _("Subscribe to the selected group(s)"), grouplist_selected_subscribe },
	{ GNOME_APP_UI_ITEM, _("Unsubscribe from Selected"), _("Unsubscribe from the selected group(s)"), grouplist_selected_unsubscribe },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, _("Get All Article Headers"), _("Get all article headers"), grouplist_selected_download_all },
	{ GNOME_APP_UI_ITEM, _("Get New Article Headers"), _("Get new article headers"), grouplist_selected_download_new },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, _("Empty Group"), _("Remove all articles from the selected group(s)."), grouplist_selected_empty },
	GNOMEUIINFO_END
};


/* 
 * callback function 
 */
void
click_groups_column_cb (GtkCList *clist, int n)
{
	if (n)
		return;
	switch (group_mode) {
	case GROUP_SUBSCRIBED:
		group_mode = GROUP_NEW;
		break;
	case GROUP_NEW:
		group_mode = GROUP_ALL;
		break;
	case GROUP_ALL:
		group_mode = GROUP_SUBSCRIBED;
		break;
	}
	grouplist_update ();
}


/*--------------------------------------------------------------------
 * easily set the ~/.gnome/Pan sections for [alt.group.name]
 *--------------------------------------------------------------------*/
void
group_set_attrib_i (group_data *gdata, char *attrib, int value)
{
	char section[512]; /* FIXME: Will this accomodate the largest groupname? */
	
	if (gdata && gdata->name && attrib) {
		g_snprintf (section, 512, "/Pan/%s:%s/%s", Pan.active_server->name, gdata->name, attrib);
		gnome_config_set_int (section, value);
		gnome_config_sync ();
	}
}


int
group_get_attrib_i (group_data *gdata, char *attrib)
{
	char section[512]; /* FIXME: Will this accomodate the largest groupname? */
	
	if (gdata && gdata->name && attrib) {
		g_snprintf (section, 512, "/Pan/%s:%s/%s", Pan.active_server->name, gdata->name, attrib);
		return gnome_config_get_int(section);
	}
	else {
		return 0;
	}
}


void
group_add_attrib_i (group_data *gdata, char *attrib, int i)
{
	int j;

	pan_lock();
	j = group_get_attrib_i (gdata, attrib);
	group_set_attrib_i (gdata, attrib, j + i);
	pan_unlock();

	/* Make the change visible */
	grouplist_update_row (gdata);
}


int
compare_group_data (group_data *a, group_data *b)
{
	return strcmp(a->name, b->name);
}


void
add_group_to_clist (group_data *gdata)
{
	int row, i;
	char *text[4];
	GtkCListRow *clistrow;
	char *p = NULL;
	char *q = NULL;

	if ((i = group_get_attrib_i (gdata, "Total"))) {
		q = g_strdup_printf ("%d", i);
		if ((i = group_get_attrib_i (gdata, "Unread")))
			p = g_strdup_printf ("%d", i);
	}
	
	text[0] = gdata->name;
	text[1] = p;
	text[2] = q;
	text[3] = NULL;

	row = gtk_clist_append (GTK_CLIST(Pan.group_clist), text);
	g_free (p);
	g_free (q);
	
	if (gdata->flags & GROUP_ONDISK) {
		gtk_clist_set_pixtext (GTK_CLIST(Pan.group_clist), row, 0, gdata->name, 5,
				       disk_pixmap->pixmap, disk_pixmap->mask);
	}

	/* The following hack is *MUCH* faster than this:
	 * gtk_clist_set_row_data(GTK_CLIST(Pan.group_clist), row, gdata);
	 * Let's hope the gtk structure doesn't change or something
	 */
	clistrow = (GtkCListRow *) GTK_CLIST(Pan.group_clist)->row_list_end->data;
	clistrow->data = gdata;
}


void
add_subscribed_group_to_clist (group_data* gdata)
{
	if (gdata->flags & GROUP_SUBSCRIBED)
		add_group_to_clist (gdata);
}


void
add_new_group_to_clist (group_data* gdata)
{
	if (gdata->flags & GROUP_NEW)
		add_group_to_clist (gdata);
}


void
group_dialog_clicked_cb (GnomeDialog* dialog, int num)
{
	switch (num) {
	case 0:					/* OK button */
		queue_add_grouplist_download ();
	default:
		gtk_widget_destroy (GTK_WIDGET(dialog));
		break;
	}
}


void
group_free (group_data *data)
{
	g_free (data->name);
	g_free (data);
}


void
grouplist_free (GSList* list)
{
	if (list) {
		g_slist_foreach(list, (GFunc) group_free, NULL);
		g_slist_free(list);
		list = NULL;
	}
}


void
get_all_groups (void)
{
	queue_add_grouplist_download ();
}


void
choose_download_dialog_clicked_cb (GnomeDialog* dialog,
				   int button_number, gpointer gdata)
{
	switch (button_number) {
	case 0:					/* All Headers */
		queue_add_articlelist_download (gdata,
						QUEUE_ARTICLELIST_DOWNLOAD_ALL |
						QUEUE_FOREGROUND);
		break;
	case 1:					/* New Headers */
		queue_add_articlelist_download (gdata,
						QUEUE_ARTICLELIST_DOWNLOAD_SAMPLE |
						QUEUE_FOREGROUND);
		break;
	default:
		/* Cancel */
		break;
	};
	gtk_widget_destroy(GTK_WIDGET(dialog));
}


void
choose_download_dialog (group_data* gdata)
{
	GtkWidget *dialog;
	GtkWidget *label;
	char *temp;

	dialog = gnome_dialog_new (_("Select Header Download Option"),
				   _("Download All Headers"),
				   _("Sample Some Headers"),
				   GNOME_STOCK_BUTTON_CANCEL, NULL);

	gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
			    GTK_SIGNAL_FUNC (choose_download_dialog_clicked_cb),
			    gdata);

	temp = g_strdup_printf (_("Select an option for group: %s"), gdata->name);
	label = gtk_label_new (temp);
	g_free (temp);

	gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), label);

	gui_popup_draw (dialog);
}


/*---[ select_group_row ]---------------------------------------------
 * process what happens when a user double clicks on a group in the
 * group list
 *--------------------------------------------------------------------*/
void
select_group_row (GtkWidget *widget, int r, int c,
		  GdkEvent *event)
{
	group_data *gdata;
	GdkEventButton *bevent = (GdkEventButton *)event;

	if (bevent) {
		if ((bevent->button) == 1 && (bevent->type == GDK_2BUTTON_PRESS)) {
			gdata = (group_data *) gtk_clist_get_row_data(GTK_CLIST(widget), r);
			
			if (Pan.active_group == gdata) {
				gui_page_set (ARTICLELIST_PAGE, Pan.article_ctree);
				articlelist_update (gdata);
				return;
			}
			if (group_index_exists(gdata)) {
				queue_add_articlelist_download (gdata,
								QUEUE_ARTICLELIST_LOAD |
								QUEUE_FOREGROUND);
			}
			else
				choose_download_dialog (gdata);
			/* if in notebook layout, flip to the article list */
			gui_page_set (ARTICLELIST_PAGE, Pan.article_ctree);
			gui_set_title (gdata);
		}
	}
}


void
groups_clist_button_press (GtkWidget *widget, GdkEventButton *bevent)
{
	if (bevent->button != 1)
		gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button_press_event");
	
	if (bevent->button == 3)
		gtk_menu_popup(GTK_MENU(groups_clist_menu), NULL, NULL, NULL,
			       NULL, bevent->button, bevent->time);
}


void
group_unload (group_data *gdata)
{
	if (gdata == Pan.active_group) {
		articlelist_clear ();
		gui_set_title (NULL);
		Pan.active_group = NULL;
	}

}

/*---[ group_empty ]--------------------------------------------------
 * actual function that empties the group and deletes the groups
 * database file and associated gnome_config sections
 *--------------------------------------------------------------------*/
void
group_empty (group_data *gdata)
{
	char section[512];

	if (gdata) {
		g_snprintf(section, 512, "/Pan/%s:%s", Pan.active_server->name, gdata->name);

		group_unload (gdata);

		if (gnome_config_has_section (section)) {
			gnome_config_clean_section (section);
			gnome_config_sync();
		}

		db_group_index_erase (gdata);

		if (gdata->flags & GROUP_SUBSCRIBED)
			gdata->flags &= ~GROUP_ONDISK;
		else 
			gdata->flags = 0;

		articlelist_free (gdata->articlelist);
		gdata->articlelist = NULL;
	}
}


/*--------------------------------------------------------------------
 * callback for group_empty, called when a button in group_empty_dialog
 * is clicked
 *--------------------------------------------------------------------*/
void
group_empty_cb (GnomeDialog *dialog,
		int button_number, gpointer gdata)
{
	switch (button_number) {
	case 0:					/* 'yes' */
		group_empty (gdata);
		break;
	case 1:					/* 'no' */
		break;
	default:				/* cancel */
		break;
	};

//	if (group_mode == GROUP_SUBSCRIBED)
//		grouplist_update ();
//	else
//		grouplist_update_row (gdata);
	if (group_mode == GROUP_SUBSCRIBED)
		grouplist_update();

	gtk_widget_destroy (GTK_WIDGET(dialog));
}


/*--------------------------------------------------------------------
 * dialog popup asking user if they'd like to remove all the messages
 * for a given group.  called when the user tries to unsubscribe
 * or they choose the menu option to empty the group manually
 *--------------------------------------------------------------------*/
void
group_empty_dialog (group_data *gdata)
{
	GtkWidget *dialog;
	GtkWidget *label;
	gchar *temp;

	pan_lock ();
//	if (group_get_attrib_i (gdata, "Total") <= 0) {
//		group_unload (gdata);
//		if (group_mode == GROUP_SUBSCRIBED)
//			grouplist_update();
//		pan_unlock();
//		return;
//	}

	dialog = gnome_dialog_new (_("Clean group"),
				   GNOME_STOCK_BUTTON_YES,
				   GNOME_STOCK_BUTTON_NO,
				   NULL);

	gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
			    GTK_SIGNAL_FUNC (group_empty_cb),
			    gdata);

	temp = g_strdup_printf (_("Remove all messages from group: \n\n%s ?"), gdata->name);
	label = gtk_label_new (temp);
	g_free (temp);

	gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), label);
	gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
	gui_popup_draw (dialog);
	pan_unlock();
}


void
grouplist_process_selected (int type)
{
	GList *l;
	group_data *gdata;

	for (l = GTK_CLIST (Pan.group_clist)->selection; l; l = l->next) {
		gdata = (group_data *) gtk_clist_get_row_data (GTK_CLIST(Pan.group_clist),
							       GPOINTER_TO_INT(l->data));
		if (gdata) {
			switch (type) {
			case 1: /* subscribe */
				gdata->flags |= GROUP_SUBSCRIBED;
				break;
			case 2: /* unsubscribe */
				gdata->flags &= ~GROUP_SUBSCRIBED;
				group_empty_dialog (gdata);
				break;
			case 3: /* download all */
				queue_add_articlelist_download (gdata,
								QUEUE_ARTICLELIST_DOWNLOAD_ALL |
								QUEUE_BACKGROUND);
				break;
			case 4: /* download new */
				queue_add_articlelist_download (gdata,
								QUEUE_ARTICLELIST_DOWNLOAD_NEW |
								QUEUE_BACKGROUND);
				break;
			case 5: /* clean/empty */
				group_empty_dialog (gdata);
				break;
			case 6: /* group properties */
			default:
				break;
			}
		}
	}

	clist_unselect_all (Pan.group_clist);

//	if (((type == 2) || (type == 5)) && (group_mode == GROUP_SUBSCRIBED))
//		grouplist_update ();

//	grouplist_save ();
}

void grouplist_selected_subscribe (void) { grouplist_process_selected (1); }
void grouplist_selected_unsubscribe (void) { grouplist_process_selected (2); }
void grouplist_selected_download_all (void) { grouplist_process_selected (3); }
void grouplist_selected_download_new (void) { grouplist_process_selected (4); }
void grouplist_selected_empty (void) { grouplist_process_selected (5); }


void
group_clean (group_data *gdata)
{
	articlelist_free (gdata->articlelist);
	gdata->articlelist = NULL;
}


GtkWidget *
grouplist_create (void)
{
	GtkWidget *temp;

	static char *titles[] = {
		_("All Groups"),
		_("Unread"),
		_("Total")
	};

	Pan.group_clist = gtk_clist_new_with_titles (3, titles);
	gtk_widget_show (Pan.group_clist);
	temp = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(temp),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER(temp), Pan.group_clist);
	gtk_widget_show (temp);

	gtk_signal_connect (GTK_OBJECT(Pan.group_clist),
			    "select_row",
			    GTK_SIGNAL_FUNC (select_group_row), NULL);

	gtk_signal_connect (GTK_OBJECT(Pan.group_clist),
			    "unselect_row",
			    GTK_SIGNAL_FUNC (select_group_row), NULL);

	gtk_signal_connect (GTK_OBJECT(Pan.group_clist),
			    "button_press_event",
			    GTK_SIGNAL_FUNC (groups_clist_button_press), NULL);

	gtk_clist_set_column_width (GTK_CLIST(Pan.group_clist), 0, 200);
	gtk_clist_set_column_auto_resize (GTK_CLIST(Pan.group_clist), 1, TRUE);
	gtk_clist_set_column_auto_resize (GTK_CLIST(Pan.group_clist), 2, TRUE);

	gtk_clist_set_selection_mode (GTK_CLIST(Pan.group_clist),
				      GTK_SELECTION_EXTENDED);

	gtk_clist_set_column_justification (GTK_CLIST(Pan.group_clist), 0,
					    GTK_JUSTIFY_LEFT);
	gtk_clist_set_column_justification (GTK_CLIST(Pan.group_clist), 1,
					    GTK_JUSTIFY_LEFT);

	gtk_signal_connect (GTK_OBJECT(Pan.group_clist), "click_column",
			    GTK_SIGNAL_FUNC(click_groups_column_cb), NULL);

	group_mode = gnome_config_get_int ("/Pan/State/group_mode=0");

	/* Create the Popup Menu while we are here. */
	groups_clist_menu = gnome_popup_menu_new (groups_menu_popup);	

	/* Lets have the Pixmap ready too */
	disk_pixmap = (GnomePixmap *) gnome_pixmap_new_from_xpm_d (disk_xpm);

	return temp;
}


/*--------------------------------------------------------------------
 * grep the group list for whatever the user typed into Find: when the
 * user hits enter in the Find: box
 *--------------------------------------------------------------------*/
void
grouplist_grep (GtkWidget *group_entry)
{
	char *entry_text;
	GSList *pos;
	group_data *gdata;

	g_return_if_fail (Pan.group_clist);

	gtk_clist_clear (GTK_CLIST(Pan.group_clist));
	entry_text = gtk_entry_get_text (GTK_ENTRY(group_entry));
	gtk_clist_freeze (GTK_CLIST(Pan.group_clist));
	pos = Pan.grouplist;
	while (pos != NULL) {
		gdata = pos->data;
		if (((gdata->flags & group_mode) ||
		     (group_mode == GROUP_ALL)) && strstr(gdata->name, entry_text))
			add_group_to_clist(pos->data);

		pos = pos->next;
	}
	gtk_clist_thaw (GTK_CLIST(Pan.group_clist));
}


void
group_save (group_data *gdata, FILE *fp)
{
	fprintf (fp, "%d\n%s\n", gdata->flags, gdata->name);
}


/*--------------------------------------------------------------------
 * load the list of groups from the 'active' file
 *--------------------------------------------------------------------*/
/* set GROUP_ONDISK if group_index_exists () is TRUE */
void
grouplist_load (void)
{
	GnomeDialog *dialog;
	GtkWidget *label;
	group_data *gdata;
	GSList *prev_list = NULL;
	GSList *temp_list;
	FILE *group_file;
	char path[PATH_MAX];
	
	g_return_if_fail (Pan.active_server && Pan.active_server->name);

	g_snprintf (path, 256, "/%s/%s/active", data_dir, Pan.active_server->name);

	if ((group_file = fopen(path, "r")) == NULL) {
		dialog = (GnomeDialog *) gnome_dialog_new (_("No Group List"), GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO, NULL);
		label = gtk_label_new (_("We don't have a group list for the active server.\nShall we get one?\n"));
		gtk_container_add (GTK_CONTAINER(dialog->vbox), label);
		gtk_signal_connect (GTK_OBJECT(dialog), "clicked", GTK_SIGNAL_FUNC(group_dialog_clicked_cb), NULL);

		gui_popup_draw (GTK_WIDGET(dialog));
		return;
	}

	while (1) {
		gdata = (group_data *) g_malloc0 (sizeof(group_data));

		fscanf (group_file, "%d\n", &gdata->flags);
		if (feof(group_file))
			break;
		gdata->name = (gchar *) fgetline (group_file);

/* FIXME: seems kind of resource heavy, but it works */
/* version #1: heaviest, 5.19s on time
		if (group_index_exists (gdata))
			gdata->flags |= GROUP_ONDISK;
*/
/* version #2: less heavy, about 4.22s on time
 		if (group_get_attrib_i (gdata, "Total") > 0)
	 		gdata->flags |= GROUP_ONDISK;
*/
		temp_list = g_slist_alloc();

		if (Pan.grouplist == NULL)
			Pan.grouplist = temp_list;
		
		temp_list->data = gdata;

		g_slist_concat (prev_list, temp_list);
		prev_list = temp_list;
	}

	fclose (group_file);
}


/*--------------------------------------------------------------------
 * save the list of groups to the 'active' file
 *--------------------------------------------------------------------*/
void
grouplist_save (void)
{
	FILE *fp;
	char path[PATH_MAX];

	g_return_if_fail (Pan.grouplist);

	g_snprintf (path, 256, "/%s/%s/active", data_dir,
		    Pan.active_server->name);

	if ((fp = fopen(path, "w")) != NULL) {
		g_slist_foreach (Pan.grouplist, (GFunc) group_save, fp);
		fclose (fp);
	}
	else
		pan_error_dialog (_("We couldn't save group list to %s:\n%s"), path,
				  strerror(errno));
}


void
grouplist_update (void)
{
	if (Pan.grouplist == NULL)
		grouplist_load();

	pan_lock ();
	gtk_clist_clear (GTK_CLIST(Pan.group_clist));
	switch (group_mode) {
	case GROUP_ALL:
		gtk_clist_set_column_title (GTK_CLIST(Pan.group_clist), 0, _("All Groups"));
		gtk_clist_freeze (GTK_CLIST(Pan.group_clist));
		g_slist_foreach (Pan.grouplist, (GFunc) add_group_to_clist, NULL);
		gtk_clist_thaw (GTK_CLIST(Pan.group_clist));
		break;
	case GROUP_SUBSCRIBED:
		gtk_clist_set_column_title (GTK_CLIST(Pan.group_clist), 0, _("Subscribed Groups"));
		gtk_clist_freeze (GTK_CLIST(Pan.group_clist));
		g_slist_foreach (Pan.grouplist, (GFunc) add_subscribed_group_to_clist, NULL);
		gtk_clist_thaw (GTK_CLIST(Pan.group_clist));
		break;
	case GROUP_NEW:
		gtk_clist_set_column_title (GTK_CLIST(Pan.group_clist), 0, _("New Groups"));
		gtk_clist_freeze (GTK_CLIST(Pan.group_clist));
		g_slist_foreach (Pan.grouplist, (GFunc) add_new_group_to_clist, NULL);
		gtk_clist_thaw (GTK_CLIST(Pan.group_clist));
		break;
	default:
		g_warning ("Reached default in grouplist_update switch\n");
	}
	pan_unlock ();
}


/*--------------------------------------------------------------------
 * called when a user double clicks on a group name, updates the clist
 * to reflect total messages and if the group is "on disk" (stored in
 * a database file)
 *--------------------------------------------------------------------*/
void
grouplist_update_row (group_data * gdata)
{
	gint row, i;
	char *p = NULL;
	
	pan_lock ();
	row = gtk_clist_find_row_from_data (GTK_CLIST(Pan.group_clist), gdata);

	gtk_clist_freeze (GTK_CLIST (Pan.group_clist));

	if ((i = group_get_attrib_i (gdata, "Total"))) {
		p = g_strdup_printf ("%d", i);
		gtk_clist_set_text (GTK_CLIST(Pan.group_clist), row, 2, p);
		g_free (p);
		
		i = group_get_attrib_i (gdata, "Unread");
		p = g_strdup_printf ("%d", i);
		gtk_clist_set_text (GTK_CLIST(Pan.group_clist), row, 1, p);
		g_free (p);
	}

	if (gdata->flags & GROUP_ONDISK)
		gtk_clist_set_pixtext (GTK_CLIST(Pan.group_clist), row, 0, gdata->name, 5,
				       disk_pixmap->pixmap, disk_pixmap->mask);
	else
		gtk_clist_set_text (GTK_CLIST(Pan.group_clist), row, 0, gdata->name);

	gtk_clist_thaw (GTK_CLIST (Pan.group_clist));

	gtk_widget_queue_draw (GTK_WIDGET(Pan.group_clist));
	pan_unlock ();
}
