/*
 * Pan - A Newsreader for X
 * Copyright (c) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <gnome.h>

#include "pan.h"

/*
 * global variables
 */
GtkWidget *notebook;


/*---[ gui_construct_notebook ]---------------------------------------
 * build and pack the notebook GUI style layout
 *--------------------------------------------------------------------*/
void
gui_notebook_construct (void)
{
	if (viewmode == GUI_NOTEBOOK) return;
	if (viewmode == GUI_PANED)
	{
		gtk_widget_ref (groups_vbox);
		gtk_container_remove (GTK_CONTAINER (groups_vbox->parent), groups_vbox);
		gtk_widget_ref (articlelist_ctree);
		gtk_container_remove (GTK_CONTAINER (articlelist_ctree->parent), articlelist_ctree);
		gtk_widget_ref (text_table);
		gtk_container_remove (GTK_CONTAINER (text_table->parent), text_table);
		gtk_container_foreach (GTK_CONTAINER (contents_vbox), (GtkCallback) gtk_widget_destroy, NULL);
	}

	notebook = gtk_notebook_new ();
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
				  groups_vbox,
				  gtk_label_new (_("Groups")));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
				  articlelist_ctree,
				  gtk_label_new (_("Articles")));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
				  text_table,
				  gtk_label_new (_("Message")));
	gtk_box_pack_start (GTK_BOX (contents_vbox), notebook, TRUE, TRUE, 0);

//	infobar = gui_create_infobar ();
//	gtk_box_pack_end (GTK_BOX (contents_vbox), infobar, FALSE, FALSE, 0);

	gtk_notebook_set_tab_border (GTK_NOTEBOOK (notebook), 2);

/* connect signals */
	gtk_signal_connect (GTK_OBJECT (Pan.article_ctree), "key_press_event",
			    GTK_SIGNAL_FUNC (gui_key_press_cb), NULL);
	gtk_signal_connect (GTK_OBJECT (Pan.group_clist), "key_press_event",
			    GTK_SIGNAL_FUNC (gui_key_press_cb),  NULL);
	gtk_signal_connect (GTK_OBJECT (text), "key_press_event",
			    GTK_SIGNAL_FUNC (gui_key_press_cb), NULL);

	gui_notebook_restore_column_widths (GTK_CLIST (Pan.group_clist), "group");
	gui_notebook_restore_column_widths (GTK_CLIST (Pan.article_ctree), "articlelist");

	if (viewmode == GUI_PANED)
	{
		gtk_widget_unref (groups_vbox);
		gtk_widget_unref (articlelist_ctree);
		gtk_widget_unref (text_table);
	}

	viewmode = GUI_NOTEBOOK;
	gtk_widget_show_all (GTK_WIDGET(contents_vbox));
}


void
gui_notebook_restore_column_widths (GtkCList *clist, gchar *type)
{
	char buf[256];
	int i;
	int tmp_width;
	int cols = GTK_CLIST (clist)->columns;

	for (i = 0; i < cols; i++) {
		if (i == 0)
			g_snprintf (buf, 256, "/Pan/Geometry/%s_column_0=200", type);
		else
			g_snprintf (buf, 256, "/Pan/Geometry/%s_column_%d", type, i);
		tmp_width = gnome_config_get_int (buf);
		gtk_clist_set_column_width (GTK_CLIST (clist), i, tmp_width);
	}
}


void
gui_notebook_save_column_widths (GtkCList *clist, gchar *type)
{
	char buf[256];
	int i;
	int cols = GTK_CLIST (clist)->columns;
	
	for (i = 0; i < cols; i++) {
		g_snprintf (buf, 256, "/Pan/Geometry/%s_column_%d", type, i);
		gnome_config_set_int (buf, (int) GTK_CLIST (clist)->column[i].width);
	}
}
