/*
 * Pan - A Newsreader for X
 * Copyright (c) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <gnome.h>
#include "pan.h"

/*---[ gui_construct_paned ]------------------------------------------
 * build the paned GUI layout
 *--------------------------------------------------------------------*/
void
gui_paned_construct (void)
{
	GtkWidget *vpane;
	GtkWidget *hpane;

	if (viewmode == GUI_PANED) return;
	if (viewmode == GUI_NOTEBOOK)
	{
		gtk_widget_ref (groups_vbox);
		gtk_container_remove (GTK_CONTAINER (groups_vbox->parent), groups_vbox);
		gtk_widget_ref (articlelist_ctree);
		gtk_container_remove (GTK_CONTAINER (articlelist_ctree->parent), articlelist_ctree);
		gtk_widget_ref (text_table);
		gtk_container_remove (GTK_CONTAINER (text_table->parent), text_table);
		gtk_container_foreach (GTK_CONTAINER (contents_vbox), (GtkCallback) gtk_widget_destroy, NULL);
	}

	hpane = gtk_hpaned_new ();
	gtk_paned_add1 (GTK_PANED(hpane), groups_vbox);
	gtk_paned_add2 (GTK_PANED(hpane), articlelist_ctree);

	vpane = gtk_vpaned_new ();
	gtk_paned_add1 (GTK_PANED(vpane), hpane);
	gtk_paned_add2 (GTK_PANED(vpane), text_table);
	gtk_box_pack_start (GTK_BOX (contents_vbox), vpane, TRUE, TRUE, 0);

//	infobar = gui_create_infobar ();
//	gtk_box_pack_end (GTK_BOX (contents_vbox), infobar, FALSE, FALSE, 0);

	if (viewmode == GUI_NOTEBOOK)
	{
		gtk_widget_unref (groups_vbox);
		gtk_widget_unref (articlelist_ctree);
		gtk_widget_unref (text_table);
	}

	viewmode = GUI_PANED;
	gtk_widget_show_all (GTK_WIDGET(contents_vbox));
}


/*--------------------------------------------------------------------
 * enter key was pressed while in paned mode
 *--------------------------------------------------------------------*/
int
gui_paned_get_current_pane (void)
{
	if (GTK_WIDGET_HAS_FOCUS (Pan.group_clist))
		return 0;
	if (GTK_WIDGET_HAS_FOCUS (Pan.article_ctree))
		return 1;
	if (GTK_WIDGET_HAS_FOCUS (text))
		return 2;
	
	return -1;
}


void
gui_paned_enter_clicked (void)
{
	int cur_pane;

	cur_pane = gui_paned_get_current_pane();

	switch (cur_pane) {
	case 0: 
		g_print ("groups pane");
		break;
	case 1: 
		g_print ("articlelist pane");
		break;
	case 2:
		g_print ("message pane");
		break;
	default:
		break;
	}
}
