/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <gnome.h>

#include "pan.h"

/* 'File' menu */
static GnomeUIInfo file_menu[] = {
	GNOMEUIINFO_MENU_PRINT_ITEM (not_implemented, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_EXIT_ITEM(pan_shutdown, NULL),
	GNOMEUIINFO_END
};

/* 'Edit' menu */
static GnomeUIInfo edit_menu[] = {
	GNOMEUIINFO_MENU_CUT_ITEM (not_implemented, NULL),
	GNOMEUIINFO_MENU_COPY_ITEM (not_implemented, NULL),
	GNOMEUIINFO_MENU_PASTE_ITEM (not_implemented, NULL),
	GNOMEUIINFO_MENU_SELECT_ALL_ITEM (not_implemented, NULL),
	GNOMEUIINFO_MENU_CLEAR_ITEM (not_implemented, NULL),
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, _("Find"), _("Find text in displayed header fields"), articlelist_find_text_cb,
	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_SEARCH, 'F', GDK_CONTROL_MASK, NULL},
	{ GNOME_APP_UI_ITEM, _("Find Next"), _("Repeat the last search."), articlelist_find_next_cb,
	  NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'G', GDK_CONTROL_MASK, NULL},
	GNOMEUIINFO_END
};

/* 'Settings' menu */
static GnomeUIInfo settings_menu[] = {
	GNOMEUIINFO_MENU_PREFERENCES_ITEM(props_spawn, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo servers_menu[] = {
	GNOMEUIINFO_END
};

/* 'Online' menu */
static GnomeUIInfo online_menu[] = {
	{ GNOME_APP_UI_ITEM, _("Go Online"), _("Go into online mode."), go_online, 0 },
	{ GNOME_APP_UI_ITEM, _("Go Offline"), _("Go into offline mode."), go_offline_mf },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, _("Stop all Online Tasks"), _("Stop all online tasks."), stop_all_mf,
	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_STOP, 'S', GDK_CONTROL_MASK, NULL},
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, _("Get List of Groups"), _("Refresh the listing of groups."), get_all_groups,
	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_REFRESH, 0, 0, NULL},
	GNOMEUIINFO_END
};

/* 'Help' menu */
GnomeUIInfo help_menu[] = {
	GNOMEUIINFO_ITEM_STOCK (_("About..."), _("Display program information, version number, and copyright."),
				gui_about, GNOME_STOCK_MENU_ABOUT),
	GNOMEUIINFO_END
};

/* 'Windows' menu */
static GnomeUIInfo view_menu[] = {
	{ GNOME_APP_UI_ITEM, _("_Notebook Layout"), _("Switch to the notebook layout."), gui_notebook_construct },
	{ GNOME_APP_UI_ITEM, _("_Paned Layout"), _("Switch to the paned layout."), gui_paned_construct },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, _("_Log Viewer"), _("Open up the Log Viewer."), log_viewer_spawn,
	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PREFERENCES, 'L', GDK_CONTROL_MASK, NULL},
//	{ GNOME_APP_UI_ITEM, _("Queue Manager"), _("Open up the Queue Manager."), queue_manager_spawn },
	GNOMEUIINFO_END
};

/* 'Group' menu */
static GnomeUIInfo group_menu[] = {
	{ GNOME_APP_UI_ITEM, _("Get All Article Headers"), _("Get all article headers for selected groups."), grouplist_selected_download_all },
	{ GNOME_APP_UI_ITEM, _("Get New Article Headers"), _("Get new article headers for selected groups."), grouplist_selected_download_new },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, _("Properties"), _("Set the properties for the selected group."), not_implemented,
	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PREFERENCES, 0, 0, NULL},
	GNOMEUIINFO_END
};

/* 'Message' menu */
static GnomeUIInfo message_menu[] = {
	{ GNOME_APP_UI_ITEM, _("New Message"), _("Create a new message."), post_window_spawn,
	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL, 'N', GDK_CONTROL_MASK, NULL },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, _("Reply to Sender"), _("Create a reply to the sender."), followup_window },
	{ GNOME_APP_UI_ITEM, _("Reply to All"), _("Post a response to everyone"), followup_window },
	{ GNOME_APP_UI_ITEM, _("Reply to Group"), _("Post a responsefollowup message"), followup_window },
	{ GNOME_APP_UI_ITEM, _("Forward"), _("Forward this message to someone."), not_implemented },
	{ GNOME_APP_UI_ITEM, _("Forward as Attachment"), _("Forward this message as an attachment to someone."), not_implemented },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, _("Decode Message"), _("Download and decode attachments in the selected messages."), articlelist_selected_decode,
	  NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'D', GDK_CONTROL_MASK, NULL},
	{ GNOME_APP_UI_ITEM, _("Delete Message"), _("Deletes the selected message from the database."), article_delete },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, _("Select All"), _("Select all messages"), articlelist_select_all,
	  NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'A', GDK_CONTROL_MASK, NULL},
	GNOMEUIINFO_END
};

static GnomeUIInfo pan_main_menu[] = {
	GNOMEUIINFO_MENU_FILE_TREE (file_menu),
	GNOMEUIINFO_MENU_EDIT_TREE(edit_menu),
	GNOMEUIINFO_MENU_SETTINGS_TREE (settings_menu),
	GNOMEUIINFO_SUBTREE(_("_View"), &view_menu),
	GNOMEUIINFO_SUBTREE(_("Servers"), &servers_menu),
	GNOMEUIINFO_SUBTREE(_("_Online"), &online_menu),
	GNOMEUIINFO_SUBTREE(_("_Group"), &group_menu),
	GNOMEUIINFO_SUBTREE(_("_Message"), &message_menu),
	GNOMEUIINFO_MENU_HELP_TREE(help_menu),
	GNOMEUIINFO_END
};

/* 
 * global variables
 */

GtkWidget *appbar;
GtkWidget *progress;

GtkWidget *infobar;
guint context_id;

GtkWidget *groups_vbox;
GtkWidget *articlelist_ctree;
GtkWidget *text_table;
GtkWidget *contents_vbox;

int viewmode = 0; /* 1 = paned, 2 = notebook */


/*---[ gui_about ]----------------------------------------------------
 * informative about window popup
 *--------------------------------------------------------------------*/
void
gui_about (void)
{
	const gchar *authors[] = {
		"Matt Eagleson <e.Messiah@superpimp.org>",
		"Jason Leach <leach@wam.umd.edu>",
		NULL
	};

	GtkWidget *about = gnome_about_new ("Pan", VERSION,
					    _("Copyright e.Messiah (C) 1999"),
					    authors,
					    "The Pimp Ass Newsreader (PAN)",
					    NULL);
	gtk_widget_show (about);
}


void
gui_clean (void)
{
	pan_lock ();
	gnome_appbar_clear_stack (GNOME_APPBAR(appbar));
	if (progress) {
		gtk_progress_set_activity_mode (GTK_PROGRESS(progress), FALSE);
		progress = NULL;
	}
	gnome_appbar_set_progress (GNOME_APPBAR(appbar), 0);
	pan_unlock ();
}


void
gui_create_appbar (GtkWidget* window)
{
	appbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_USER);
	gnome_appbar_set_default (GNOME_APPBAR(appbar), _("Pan - Offline"));
	gnome_app_set_statusbar (GNOME_APP(window), appbar);
}


GtkWidget*
gui_create_infobar (void)
{
	GtkWidget *newbar;

	newbar = gtk_statusbar_new ();
	context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR(newbar), "pan");
	return newbar;
}


void
gui_set_status (gchar *msg)
{
	pan_lock ();
	gnome_appbar_pop (GNOME_APPBAR(appbar));
	gnome_appbar_push (GNOME_APPBAR(appbar), msg);
	pan_unlock ();
}


void
gui_set_status_default (gchar *msg)
{
	pan_lock ();
	gnome_appbar_set_default (GNOME_APPBAR(appbar), msg);
	pan_unlock ();
}


void
gui_set_status_va (const gchar * format, ...)
{
	va_list args;
	gchar *msg;

	g_return_if_fail (format != NULL);

	va_start (args, format);
	msg = g_strdup_vprintf (format, args);
	va_end (args);

	pan_lock ();
	gnome_appbar_pop (GNOME_APPBAR(appbar));
	gnome_appbar_push (GNOME_APPBAR(appbar), msg);
	pan_unlock ();
	g_free (msg);
}


/*
 * set the progress bar to a percentage
 */
void
gui_set_progress (gfloat percent)
{
	pan_lock ();
	if ((percent >= 0) && (percent <= 1))
		gnome_appbar_set_progress (GNOME_APPBAR(appbar), percent);
	pan_unlock ();
}


/* ??? */
void
gui_set_progress_activity (gint count)
{
	pan_lock ();
	if (progress == NULL) {
		progress = GTK_WIDGET (gnome_appbar_get_progress (GNOME_APPBAR(appbar)));
		gtk_progress_set_activity_mode (GTK_PROGRESS(progress), TRUE);
		gtk_progress_bar_set_discrete_blocks (GTK_PROGRESS_BAR(progress), 10);
		gtk_progress_bar_set_activity_step (GTK_PROGRESS_BAR(progress), 1);
	}
	gtk_progress_set_value (GTK_PROGRESS(progress), (gfloat) (count % 11));
	pan_unlock ();
}


/*---[ gui_set_info ]-------------------------------------------------
 * set the infobar to a single message
 *--------------------------------------------------------------------*/
void
gui_set_info (gchar* msg)
{
	pan_lock ();
	gtk_statusbar_pop (GTK_STATUSBAR(infobar), context_id);
	gtk_statusbar_push (GTK_STATUSBAR(infobar), context_id, msg);
	pan_unlock ();
}


/*---[ gui_set_info_va ]----------------------------------------------
 * set the infobar to a message (variable arguments)
 *--------------------------------------------------------------------*/
void
gui_set_info_va (const gchar *format, ...)
{
	va_list args;
	gchar *msg;

	g_return_if_fail (format != NULL);

	va_start (args, format);
	msg = g_strdup_vprintf (format, args);
	va_end (args);

	pan_lock ();
	gtk_statusbar_pop (GTK_STATUSBAR(infobar), context_id);
	gtk_statusbar_push (GTK_STATUSBAR(infobar), context_id, msg);
	pan_unlock ();
	g_free (msg);
}


/*---[ gui_shutdown ]-------------------------------------------------
 * save geometry states into config files
 *--------------------------------------------------------------------*/
void
gui_shutdown (void)
{
	int x=0, y=0, width=640, height=400;

	gdk_window_get_position (Pan.window->window, &x, &y);
	gdk_window_get_size (Pan.window->window, &width, &height);
				
	gnome_config_set_int ("/Pan/Geometry/pan_x", x);
	gnome_config_set_int ("/Pan/Geometry/pan_y", y);
	gnome_config_set_int ("/Pan/Geometry/width", width);
	gnome_config_set_int ("/Pan/Geometry/height", height);
	gnome_config_set_int ("/Pan/State/viewmode", viewmode);
	gnome_config_set_int ("/Pan/State/group_mode", group_mode);
	if (viewmode == GUI_NOTEBOOK) {
		gnome_config_set_int ("/Pan/State/page", gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)));
		gui_notebook_save_column_widths (GTK_CLIST (Pan.group_clist), "group");
		gui_notebook_save_column_widths (GTK_CLIST (Pan.article_ctree), "articlelist");
	}

	gnome_config_sync();
}


/*---[ gui_construct ]------------------------------------------------
 * initialize and place the basic interface components
 *--------------------------------------------------------------------*/
void
gui_construct (void)
{
	GtkWidget *w, *x;
	/* int geo_x=0, geo_y=0; */

	Pan.window = gnome_app_new ("Pan", "Pan");
	gtk_signal_connect (GTK_OBJECT(Pan.window), "delete_event", 
			    GTK_SIGNAL_FUNC (pan_shutdown), NULL);

	gtk_window_set_default_size (GTK_WINDOW(Pan.window), 
				     gnome_config_get_int ("/Pan/Geometry/width=640"),
				     gnome_config_get_int ("/Pan/Geometry/height=400"));

/* Groups vbox */
	groups_vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_set_usize (GTK_WIDGET(groups_vbox), 300, 200);
	x = gtk_hbox_new (FALSE, 0);
	w = gtk_label_new ("Find:");
	gtk_box_pack_start (GTK_BOX (x), w, FALSE, FALSE, 3);
	w = gtk_entry_new ();
	gtk_signal_connect (GTK_OBJECT (w), "activate",
			    GTK_SIGNAL_FUNC(grouplist_grep), w);
	gtk_box_pack_start (GTK_BOX (x), w, TRUE, TRUE, 3);

	gtk_box_pack_start (GTK_BOX (groups_vbox), grouplist_create(), TRUE, TRUE, 1);
	gtk_box_pack_start (GTK_BOX (groups_vbox), x, FALSE, FALSE, 1);

/* Articlelist_ctree */
	w = create_articlelist_ctree ();
	articlelist_ctree = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (articlelist_ctree), w, TRUE, TRUE, 0);

/* Text table */
	w = text_create ();
	text_table = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (text_table), w, TRUE, TRUE, 0);

	gui_create_appbar (Pan.window);

	gnome_app_create_menus (GNOME_APP(Pan.window), pan_main_menu);
	gnome_app_install_menu_hints (GNOME_APP(Pan.window), pan_main_menu);

	contents_vbox = gtk_vbox_new (FALSE, 0);
	gnome_app_set_contents (GNOME_APP (Pan.window), contents_vbox);

	if (gnome_config_get_int("Pan/State/viewmode=1") == 2)
		gui_notebook_construct ();
	else
		gui_paned_construct ();

/* FIXME: this isn't useful until we can get it to automatically load the last
   selected group or message.  Otherwise, it remembers you were in the article
   list view, but then it's just blank, so you gotta switch back to the group
   list mode anyhow.
 	if (viewmode == GUI_NOTEBOOK) {
	 	int last_page = gnome_config_get_int ("/Pan/State/page=0");
		 tk_notebook_set_page (GTK_NOTEBOOK (notebook), last_page);
	}
*/

	/* FIXME: 0,0 shows up under the panel for me on IceWm
	 * I'm not sure we should mess with window placement
	 * until the user tells us to... 
	 geo_x = gnome_config_get_int ("/Pan/Geometry/pan_x=0");
	 geo_y = gnome_config_get_int ("/Pan/geometry/pan_y=0");
	 gdk_window_move (Pan.window->window, geo_x, geo_y);
	*/
}


/*---[ gui_page_set ]-------------------------------------------------
 * flip to a different notebook page, setting keyboard focus on a 
 * widget in the new page
 *--------------------------------------------------------------------*/
void
gui_page_set (int page, GtkWidget *focus_item)
{
	if (viewmode == GUI_NOTEBOOK) {
		gtk_notebook_set_page (GTK_NOTEBOOK (notebook), page);
		gtk_widget_grab_focus (focus_item);
	}
}


/*---[ gui_page_change ]----------------------------------------------
 * flip to a differnt notebook page, setting keyboard focus in new page.  
 * This is different from gui_page_set because it flips the page based on 
 * a offset, like -1 or 1, to flip back one page or forward one.  It is 
 * automatic in handling the focus grabbing too.
 *--------------------------------------------------------------------*/
void
gui_page_change (int change)
{
	int cur_page;
	article_data *adata;
	group_data *gdata;

	if (viewmode == GUI_PANED)
		cur_page = gui_paned_get_current_pane ();
	if (viewmode == GUI_NOTEBOOK)
		cur_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
	else
		return;

	cur_page += change;
	if ((cur_page < 0) || (cur_page > 2)) 
		return;

	if (change < 0) { /* Escape was pressed */
		switch (cur_page) {
		case GROUPS_PAGE:
			gtk_widget_grab_focus (Pan.group_clist);
			break;
		case ARTICLELIST_PAGE:
			gtk_widget_grab_focus (Pan.article_ctree);
			break;
		case MESSAGE_PAGE:
			gtk_widget_grab_focus (text);
			break;
		default:
			break;
		}
	}
	else { /* enter key hit */
/* FIXME: segfaults if selection is zero, but when user hits escape, it
   automatically sends a "undo_selection" signal, deselecting all */
		switch (cur_page) {
		case GROUPS_PAGE:
			gtk_widget_grab_focus (Pan.group_clist);
			break;
		case ARTICLELIST_PAGE:
//			if (!GTK_CLIST (Pan.group_clist)->selection && !GTK_CLIST (Pan.group_clist)->undo_selection)
//				return;

			gdata = (group_data *) gtk_clist_get_row_data (GTK_CLIST (Pan.group_clist), 
								       GPOINTER_TO_INT (GTK_CLIST (Pan.group_clist)->selection->data));
			if (gdata) {
				if (Pan.active_group == gdata) {
					gtk_widget_grab_focus (Pan.article_ctree);
					break;
				}
				if (group_index_exists(gdata))
					queue_add_articlelist_download (gdata,
									QUEUE_ARTICLELIST_LOAD |
									QUEUE_FOREGROUND);
				else
					choose_download_dialog (gdata);
			}

			gtk_widget_grab_focus (Pan.article_ctree);
			gui_set_title (gdata);
			break;
		case MESSAGE_PAGE:
			adata = (article_data *) gtk_ctree_node_get_row_data (GTK_CTREE (Pan.article_ctree), 
									      GTK_CLIST (Pan.article_ctree)->selection->data);
			if (adata) {
				current_article = adata;
				if (adata->body == NULL) {
					queue_add_article_download (adata,
								    QUEUE_ARTICLE_DOWNLOAD |
								    QUEUE_FOREGROUND);
					adata->state |= STATE_DOWNLOAD_QUEUED;
					articlelist_update_node (adata);
				}
				else
					text_update (adata);
			}
			gtk_widget_grab_focus (text);
			break;
		default:
			break;
		}
	}

	if (viewmode == GUI_NOTEBOOK)
		gtk_notebook_set_page (GTK_NOTEBOOK (notebook), cur_page);
/*
FIXME: this would be nice, there is a slight problem with the tabs at the top
being refreshed after repeated flipping around.  But redrawing the entire
notebook looks flickery and will waste a lot of resources.  Maybe redraw at
some other less frequent place or redraw only the tab elements (if that is 
possible).
	gtk_widget_draw (GTK_WIDGET (notebook), NULL);
*/
}


void
gui_enter_clicked (void)
{
	if (viewmode == GUI_NOTEBOOK)
		gui_page_change (1);
	else if (viewmode == GUI_PANED)
		gui_paned_enter_clicked ();
}


/*---[ gui_set_title ]------------------------------------------------
 * set Pan's title to reflect the current open group
 *--------------------------------------------------------------------*/
void
gui_set_title (group_data *gdata)
{
	gchar *buf;
	
	if (gdata) {
			   buf = g_strdup_printf ("Pan [%s]", gdata->name);
			   gtk_window_set_title (GTK_WINDOW (Pan.window), buf);
			   g_free (buf);
	}
	else
		gtk_window_set_title (GTK_WINDOW (Pan.window), "Pan");

}

/*---[ gui_key_press_cb ]---------------------------------------------
 * key press callback, when a key is pressed in the group list, article
 * list, or in the message view.  This allows better keyboard navigation
 * and the potential for new key bindings to imitate Ag*nt.
 *--------------------------------------------------------------------*/
void
gui_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data)
{
	switch (event->keyval)
	{
	case GDK_Q:
	case GDK_q:
		gui_page_change (-1);
		break;
	case GDK_Return:
		gui_page_change (1);
		break;
	default:
		break;
	}
}


/*---[ gui_popup_draw ]----------------------------------------------
 * use this to draw popup windows or dialogs, it will center them in the 
 * exact center of the Pan window, show, and raise them.
 *--------------------------------------------------------------------*/
void
gui_popup_draw (GtkWidget *dialog)
{
	int x=0, y=0, width=640, height=400;
	int dlg_width, dlg_height;
	int dlg_x, dlg_y;

/*	gtk_window_position (GTK_WINDOW (dialog), GTK_WIN_POS_NONE); */
	
	gdk_window_get_position (Pan.window->window, &x, &y);
	gdk_window_get_size (Pan.window->window, &width, &height);

	gtk_widget_realize (dialog);
	gdk_window_get_size (dialog->window, &dlg_width, &dlg_height);

	dlg_x = x + width/2 - dlg_width/2;
	dlg_y = y + height/2 - dlg_height/2;

	gtk_widget_set_uposition (dialog, dlg_x, dlg_y);
	gtk_widget_show_all (dialog);

	gdk_window_raise (dialog->window);	
}
