/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <gnome.h>
#include "pan.h"

/* global application structure */
struct _Pan Pan;


/*--------------------------------------------------------------------
 * pan_init() is when the druid finishes on Pan's first run, after 
 * that it is called right before gtk_main() is called
 *--------------------------------------------------------------------*/
void
pan_init (void)
{
	log_init ();
	props_init ();
	server_init ();
	queue_init ();
//	queue_manager_init ();
}


/*--------------------------------------------------------------------
 * _the_ main function!
 *--------------------------------------------------------------------*/
int
main (int argc, char *argv[])
{
	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
	textdomain (PACKAGE);

	g_thread_init (NULL);

	gnome_init ("Pan", VERSION, argc, argv);

	/* initialize variables in the global app structure */
	Pan.grouplist = NULL;
	Pan.active_group = NULL;
	Pan.error = NULL;

	gui_construct ();

	/* Check if a Preferences file exists. */
	if (!gnome_config_has_section("/Pan/User")) {
		gtk_widget_show_all (Pan.window);
		druid_spawn ();
	}
	else {
		pan_init ();
		gtk_widget_show_all (Pan.window);
	}

	gdk_threads_enter ();
	gtk_main ();

	return 0;
}


/*--------------------------------------------------------------------
 * shutdown Pan, closing all tcp, and saving all proper config
 * options
 *--------------------------------------------------------------------*/
void
pan_shutdown (void)
{
	tcp_close ();
	gui_set_status(_("Closing article database..."));

	if (Pan.grouplist)
		grouplist_save ();

	gui_shutdown ();
	gtk_main_quit ();
}
