/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <gnome.h>
#include <sys/stat.h>
#include <errno.h>
#include <ctype.h>
#include "pan.h"


void
pan_error_dialog (const gchar *format, ...)
{
	va_list args;
	gchar *string;

	g_return_if_fail (format != NULL);

	va_start (args, format);
	string = g_strdup_vprintf (format, args);
	va_end (args);

	pan_lock ();
	gnome_error_dialog (string);
	pan_unlock ();
	g_free (string);
}


gchar *
Pan_strtrim (gchar *string)
{
	gchar *rp, *wp;
	int ws = 1;

	for (wp = rp = string; *rp; rp++) {	/* loop over string */
		if (isspace(*rp)) {		/* is it a white space? */
			if (!ws) {		/* was the last character not a white space? */
				*wp++ = ' ';	/* store a blank */
				ws = 1;
			}
		} else {			/* no white space */
			*wp++ = *rp;		/* store the character */
			ws = 0;
		}
	}
	if (ws)					/* delete trailing blank */
		wp--;

	*wp = '\0';

	return (string);
}


gchar *
Pan_strbrk (gchar *str, gchar *list)
{
	gchar ch, ch1, *p;

	while ((ch = *str) != 0) {
		p = list;
		while ((ch1 = *p) != 0) {
			if (ch == ch1)
				return str;
			p++;
		}
		str++;
	}
	return NULL;
}


/*---[ not_implemented ]----------------------------------------------
 * ok dialog popup to notify user that we are so lazy we haven't
 * added their feature yet.
 *--------------------------------------------------------------------*/
void
not_implemented (void)
{
	gnome_ok_dialog (_("Not yet implemented."));
}


void
clist_unselect_all (GtkWidget *clist)
{
	GList *list;
	gint row;

	list = GTK_CLIST(clist)->selection;
	while (list) {
		row = GPOINTER_TO_INT (list->data);
		list = list->next;
		gtk_clist_unselect_row (GTK_CLIST(clist), row, 0);
	}
	g_list_free (list);
}


gchar *
fgetline (FILE *fp)
{
	gchar *tmp = NULL;
	long pos;
	gint i;

	/* Skip Whitespace */
	for (; fgetc(fp) == ' ';);
	fseek(fp, -1, SEEK_CUR);

	/* Record String start */
	pos = ftell(fp);

	/* Measure size of string */
	for (i = 1; fgetc(fp) != '\n'; i++);

	if (i == 1)
		return NULL;

	/* Rewind to beginning of string */
	if ((fseek(fp, pos, SEEK_SET)) == -1)
		return NULL;

	/* Make room and grab that string */
	tmp = (gchar *) g_malloc (i);
	fgets (tmp, i, fp);

	/* leave the file past the newline */
	fseek (fp, 1, SEEK_CUR);

	return tmp;
}


void
pan_lock (void)
{
	if (pthread_self() == Pan.thread)
		gdk_threads_enter();
}


void
pan_unlock (void)
{
	if (pthread_self() == Pan.thread)
		gdk_threads_leave();
}


void
directory_check (gchar * pathname)
{
	struct stat st;
	gchar *dir;
	gint len, i;

	len = strlen(pathname);

	i = 1;
	while (i <= len) {
		if (pathname[i] == '/' || pathname[i] == '\0') {
			dir = g_strndup(pathname, i);
			if (stat(dir, &st) != 0) {
				int ret;
				ret = mkdir(dir, S_IRUSR | S_IWUSR | S_IXUSR);
				if (ret == -1) {
					pan_error_dialog (_("Couldn't create directory: "),
							  pathname, strerror(errno));
					return;
				}
			}
		}
		i++;
	}
}
